#!/usr/bin/perl

#
#	This perl code demonstrates using a perl script
#	with Net:SSLeay (and OpenSSL) to send and receive
#	secure messages with ACH Direct's Payments Gateway.
#	It was derived from examples in the Net::SSLeay
#	documentation.
#
#	Note that this script has been tested with
#		Net::SSLeay v1.18 (www.cpan.org) and
#		OpenSSL v0.9.6g (www.OpenSSL.org)
#		RedHat Linux 7.3
#


use Socket;
use Net::SSLeay qw(die_now die_if_ssl_error) ;
Net::SSLeay::load_error_strings();
Net::SSLeay::SSLeay_add_ssl_algorithms();
Net::SSLeay::randomize();
$msg= "pg_merchant_id=100000\n".
      "pg_password=password\n".
      "pg_transaction_type=20\n".
      "pg_total_amount=1.00\n".
      "Ecom_BillTo_Postal_Name_First=John\n".
      "Ecom_BillTo_Postal_Name_Last=Smith\n".
      "Ecom_Payment_Check_AccounT_Type=S\n".
      "Ecom_Payment_Check_Account=1234567812345678\n".
      "Ecom_Payment_Check_TRN=021000021\n".
      "endofdata\n";


#($dest_serv, $port) = @ARGV;      # Read command line
$dest_serv="www.PaymentsGateway.net";
$port = 6050;
$port = getservbyname ($port, 'tcp') unless $port =~ /^\d+$/;
$dest_ip = gethostbyname ($dest_serv);
$dest_serv_params  = sockaddr_in($port, $dest_ip);

socket  (S, &AF_INET, &SOCK_STREAM, 0)  or die "socket: $!";
connect (S, $dest_serv_params)          or die "connect: $!";
select  (S); $| = 1; select (STDOUT);   # Eliminate STDIO buffering

# The network connection is now open, lets fire up SSL    

$ctx = Net::SSLeay::CTX_new() or die_now("Failed to create SSL_CTX $!");
Net::SSLeay::CTX_set_options($ctx, &Net::SSLeay::OP_ALL)
     and die_if_ssl_error("ssl ctx set options");
$ssl = Net::SSLeay::new($ctx) or die_now("Failed to create SSL $!");
Net::SSLeay::set_fd($ssl, fileno(S));   # Must use fileno
$res = Net::SSLeay::connect($ssl) and die_if_ssl_error("ssl connect");
print "Cipher `" . Net::SSLeay::get_cipher($ssl) . "'\n";

# Exchange data

$msg .="\n";
$res = Net::SSLeay::write($ssl, $msg);  # Perl knows how long $msg is
die_if_ssl_error("ssl write");
CORE::shutdown S, 1;  # Half close --> No more output, sends EOF to server

# Read data until "endofdata" found
$got="";
do {
	$got = $got.Net::SSLeay::read($ssl);    # Perl returns undef on failure
	die_if_ssl_error("ssl read");
} until (index($got,"endofdata")>0);

# Split response message into array of lines
@lines=split("\n",$got);

# Show response
for $l (@lines) {
	print "GOT:$l\n";
}
        
Net::SSLeay::free ($ssl);               # Tear down connection
Net::SSLeay::CTX_free ($ctx);
close S;
